@@ -24,6 +24,7 @@
 #endif
 
 #include "gnl.h"
+#include "gnlmarshal.h"
 
 /**
  * SECTION:element-gnloperation
@@ -67,6 +68,14 @@ enum
   ARG_SINKS,
 };
 
+enum
+{
+  INPUT_PRIORITY_CHANGED,
+  LAST_SIGNAL
+};
+
+static guint gnl_operation_signals[LAST_SIGNAL] = { 0 };
+
 static void gnl_operation_finalize (GObject * object);
 
 static void gnl_operation_set_property (GObject * object, guint prop_id,
@@ -123,6 +132,20 @@ gnl_operation_class_init (GnlOperationClass * klass)
           "Number of input sinks (-1 for automatic handling)", -1, G_MAXINT, -1,
           G_PARAM_READWRITE));
 
+  /**
+   * GnlOperation:input-priority-changed:
+   * @pad: The operation's input pad whose priority changed.
+   * @priority: The new priority
+   *
+   * Signals that the @priority of the stream being fed to the given @pad
+   * might have changed.
+   */
+  gnl_operation_signals[INPUT_PRIORITY_CHANGED] =
+      g_signal_new ("input-priority-changed", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GnlOperationClass,
+          input_priority_changed), NULL, NULL, gnl_marshal_VOID__OBJECT_UINT,
+      G_TYPE_NONE, 2, GST_TYPE_PAD, G_TYPE_UINT);
+
   gstelement_class->request_new_pad =
       GST_DEBUG_FUNCPTR (gnl_operation_request_new_pad);
   gstelement_class->release_pad = GST_DEBUG_FUNCPTR (gnl_operation_release_pad);
@@ -159,7 +182,8 @@ gnl_operation_reset (GnlOperation * operation)
 }
 
 static void
-gnl_operation_init (GnlOperation * operation, GnlOperationClass * klass)
+gnl_operation_init (GnlOperation * operation,
+    GnlOperationClass * klass G_GNUC_UNUSED)
 {
   gnl_operation_reset (operation);
   operation->ghostpad = NULL;
@@ -209,21 +233,37 @@ element_is_valid_filter (GstElement * element, gboolean * isdynamic)
   }
   gst_iterator_free (pads);
 
-  factory = gst_element_get_factory (element);
+  if (G_LIKELY ((factory = gst_element_get_factory (element)))) {
 
-  for (templates = gst_element_factory_get_static_pad_templates (factory);
-      templates; templates = g_list_next (templates)) {
-    GstStaticPadTemplate *template = (GstStaticPadTemplate *) templates->data;
+    for (templates = gst_element_factory_get_static_pad_templates (factory);
+        templates; templates = g_list_next (templates)) {
+      GstStaticPadTemplate *template = (GstStaticPadTemplate *) templates->data;
 
-    if (template->direction == GST_PAD_SRC)
-      havesrc = TRUE;
-    else if (template->direction == GST_PAD_SINK) {
-      if (!havesink && (template->presence == GST_PAD_REQUEST))
-        *isdynamic = TRUE;
-      havesink = TRUE;
+      if (template->direction == GST_PAD_SRC)
+        havesrc = TRUE;
+      else if (template->direction == GST_PAD_SINK) {
+        if (!havesink && (template->presence == GST_PAD_REQUEST) && isdynamic)
+          *isdynamic = TRUE;
+        havesink = TRUE;
+      }
+    }
+  } else if (GST_ELEMENT_GET_CLASS (element)) {
+    GList *tmp =
+        gst_element_class_get_pad_template_list (GST_ELEMENT_GET_CLASS
+        (element));
+    while (tmp) {
+      GstPadTemplate *template = (GstPadTemplate *) tmp->data;
+
+      if (template->direction == GST_PAD_SRC)
+        havesrc = TRUE;
+      else if (template->direction == GST_PAD_SINK) {
+        if (!havesink && (template->presence == GST_PAD_REQUEST) && isdynamic)
+          *isdynamic = TRUE;
+        havesink = TRUE;
+      }
+      tmp = g_list_next (tmp);
     }
   }
-
   return (havesink && havesrc);
 }
 
@@ -319,14 +359,13 @@ gnl_operation_add_element (GstBin * bin, GstElement * element)
         operation->dynamicsinks = isdynamic;
 
         /* Source ghostpad */
-        if (!operation->ghostpad) {
+        if (operation->ghostpad)
+          gnl_object_ghost_pad_set_target (GNL_OBJECT (operation),
+              operation->ghostpad, srcpad);
+        else
           operation->ghostpad =
-              gst_ghost_pad_new_no_target ("src", GST_PAD_SRC);
-          gst_pad_set_active (operation->ghostpad, TRUE);
-          gst_element_add_pad ((GstElement *) bin, operation->ghostpad);
-        }
-        gst_ghost_pad_set_target ((GstGhostPad *) operation->ghostpad, srcpad);
-        gst_object_unref (srcpad);
+              gnl_object_ghost_pad_full (GNL_OBJECT (operation),
+              GST_PAD_NAME (srcpad), srcpad, TRUE);
 
         /* Figure out number of static sink pads */
         operation->num_sinks = get_nb_static_sinks (operation);
@@ -476,7 +515,7 @@ get_unused_static_sink_pad (GnlOperation * operation)
   return ret;
 }
 
-static GstPad *
+GstPad *
 get_unlinked_sink_ghost_pad (GnlOperation * operation)
 {
   GstIterator *pads;
@@ -576,7 +615,8 @@ add_sink_pad (GnlOperation * operation)
     /* static sink pads */
     ret = get_unused_static_sink_pad (operation);
     if (ret) {
-      gpad = gst_ghost_pad_new (GST_PAD_NAME (ret), ret);
+      gpad = gnl_object_ghost_pad ((GnlObject *) operation, GST_PAD_NAME (ret),
+          ret);
       gst_object_unref (ret);
     }
   }
@@ -585,14 +625,13 @@ add_sink_pad (GnlOperation * operation)
     /* request sink pads */
     ret = get_request_sink_pad (operation);
     if (ret) {
-      gpad = gst_ghost_pad_new (GST_PAD_NAME (ret), ret);
+      gpad = gnl_object_ghost_pad ((GnlObject *) operation, GST_PAD_NAME (ret),
+          ret);
       gst_object_unref (ret);
     }
   }
 
   if (gpad) {
-    gst_pad_set_active (gpad, TRUE);
-    gst_element_add_pad ((GstElement *) operation, gpad);
     operation->sinks = g_list_append (operation->sinks, gpad);
     operation->realsinks++;
     GST_DEBUG ("Created new pad %s:%s ghosting %s:%s",
@@ -629,9 +668,11 @@ remove_sink_pad (GnlOperation * operation, GstPad * sinkpad)
     GstPad *target = gst_ghost_pad_get_target ((GstGhostPad *) sinkpad);
 
     /* release the target pad */
+    gnl_object_ghost_pad_set_target ((GnlObject *) operation, sinkpad, NULL);
     gst_element_release_request_pad (operation->element, target);
     operation->sinks = g_list_remove (operation->sinks, sinkpad);
-    gst_element_remove_pad ((GstElement *) operation, sinkpad);
+    gnl_object_remove_ghost_pad ((GnlObject *) operation, sinkpad);
+    operation->realsinks--;
   }
 
 beach:
@@ -642,10 +683,10 @@ static void
 synchronize_sinks (GnlOperation * operation)
 {
 
-  GST_DEBUG_OBJECT (operation, "num_sinks:%d , realsinks:%d",
-      operation->num_sinks, operation->realsinks);
-  if ((operation->dynamicsinks) ||
-      (operation->num_sinks == operation->realsinks))
+  GST_DEBUG_OBJECT (operation, "num_sinks:%d , realsinks:%d, dynamicsinks:%d",
+      operation->num_sinks, operation->realsinks, operation->dynamicsinks);
+
+  if (operation->num_sinks == operation->realsinks)
     return;
 
   if (operation->num_sinks > operation->realsinks) {
@@ -697,3 +738,13 @@ gnl_operation_release_pad (GstElement * element, GstPad * pad)
 
   remove_sink_pad ((GnlOperation *) element, pad);
 }
+
+void
+gnl_operation_signal_input_priority_changed (GnlOperation * operation,
+    GstPad * pad, guint32 priority)
+{
+  GST_DEBUG_OBJECT (operation, "pad:%s:%s, priority:%d",
+      GST_DEBUG_PAD_NAME (pad), priority);
+  g_signal_emit (operation, gnl_operation_signals[INPUT_PRIORITY_CHANGED],
+      0, pad, priority);
+}
